@@ -23,8 +23,6 @@
 #include <alsa/asoundlib.h>
 
 #include "alsa.h"
-#include "timecoder.h"
-#include "player.h"
 
 
 /* This structure doesn't have corresponding functions to be an
@@ -206,7 +204,7 @@ static int pcm_revents(struct alsa_pcm_t *alsa, unsigned short *revents) {
 
 /* Start the audio device capture and playback */
 
-static void start(struct device_t *dv)
+static void start(struct device *dv)
 {
     struct alsa_t *alsa = (struct alsa_t*)dv->local;
 
@@ -218,7 +216,7 @@ static void start(struct device_t *dv)
 /* Register this device's interest in a set of pollfd file
  * descriptors */
 
-static ssize_t pollfds(struct device_t *dv, struct pollfd *pe, size_t z)
+static ssize_t pollfds(struct device *dv, struct pollfd *pe, size_t z)
 {
     int total, r;
     struct alsa_t *alsa = (struct alsa_t*)dv->local;
@@ -246,18 +244,12 @@ static ssize_t pollfds(struct device_t *dv, struct pollfd *pe, size_t z)
 /* Collect audio from the player and push it into the device's buffer,
  * for playback */
 
-static int playback(struct device_t *dv)
+static int playback(struct device *dv)
 {
     int r;
     struct alsa_t *alsa = (struct alsa_t*)dv->local;
 
-    if (dv->player) {
-        player_collect(dv->player, alsa->playback.buf,
-                       alsa->playback.period, alsa->playback.rate);
-    } else {
-        memset(alsa->playback.buf, 0,
-               alsa->playback.period * DEVICE_CHANNELS * sizeof(short));
-    }    
+    device_collect(dv, alsa->playback.buf, alsa->playback.period);
 
     r = snd_pcm_writei(alsa->playback.pcm, alsa->playback.buf,
                        alsa->playback.period);
@@ -276,7 +268,7 @@ static int playback(struct device_t *dv)
 /* Pull audio from the device's buffer for capture, and pass it
  * through to the timecoder */
 
-static int capture(struct device_t *dv)
+static int capture(struct device *dv)
 {
     int r;
     struct alsa_t *alsa = (struct alsa_t*)dv->local;
@@ -290,9 +282,8 @@ static int capture(struct device_t *dv)
         fprintf(stderr, "alsa: capture underrun %d/%ld.\n",
                 r, alsa->capture.period);
     }
-    
-    if (dv->timecoder)
-        timecoder_submit(dv->timecoder, alsa->capture.buf, r);
+
+    device_submit(dv, alsa->capture.buf, r);
 
     return 0;
 }
@@ -301,7 +292,7 @@ static int capture(struct device_t *dv)
 /* After poll() has returned, instruct a device to do all it can at
  * the present time. Return zero if success, otherwise -1 */
 
-static int handle(struct device_t *dv)
+static int handle(struct device *dv)
 {
     int r;
     unsigned short revents;
@@ -372,7 +363,7 @@ static int handle(struct device_t *dv)
 }
 
 
-static unsigned int sample_rate(struct device_t *dv)
+static unsigned int sample_rate(struct device *dv)
 {
     struct alsa_t *alsa = (struct alsa_t*)dv->local;
 
@@ -382,7 +373,7 @@ static unsigned int sample_rate(struct device_t *dv)
 
 /* Close ALSA device and clear any allocations */
 
-static void clear(struct device_t *dv)
+static void clear(struct device *dv)
 {
     struct alsa_t *alsa = (struct alsa_t*)dv->local;
 
@@ -392,19 +383,18 @@ static void clear(struct device_t *dv)
 }
 
 
-static struct device_type_t alsa_type = {
+static struct device_ops alsa_ops = {
     .pollfds = pollfds,
     .handle = handle,
     .sample_rate = sample_rate,
     .start = start,
-    .stop = NULL,
     .clear = clear
 };
 
 
 /* Open ALSA device. Do not operate on audio until device_start() */
 
-int alsa_init(struct device_t *dv, const char *device_name,
+int alsa_init(struct device *dv, const char *device_name,
               int rate, int buffer_time)
 {
     struct alsa_t *alsa;
@@ -430,7 +420,7 @@ int alsa_init(struct device_t *dv, const char *device_name,
     }
 
     dv->local = alsa;
-    dv->type = &alsa_type;
+    dv->ops = &alsa_ops;
 
     return 0;
 
@@ -440,3 +430,16 @@ int alsa_init(struct device_t *dv, const char *device_name,
     free(alsa);
     return -1;
 }
+
+
+/* ALSA caches information when devices are open. Provide a call
+ * to clear these caches so that valgrind output is clean. */
+
+void alsa_clear_config_cache(void)
+{
+    int r;
+
+    r = snd_config_update_free_global();
+    if (r < 0)
+        alsa_error("config_update_free_global", r);
+}
