@@ -22,7 +22,9 @@
 
 #include "device.h"
 
-int alsa_init(struct device_t *dv, const char *name,
+int alsa_init(struct device *dv, const char *name,
               int rate, int buffer_time);
 
+void alsa_clear_config_cache(void);
+
 #endif
