@@ -4,12 +4,12 @@
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * version 2, as published by the Free Software Foundation.
- * 
+ *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  * General Public License version 2 for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * version 2 along with this program; if not, write to the Free
  * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
@@ -39,12 +39,10 @@
 #include "timecoder.h"
 #include "xwax.h"
 
-
 /* Screen refresh time in milliseconds */
 
 #define REFRESH 10
 
-
 /* Font definitions */
 
 #define FONT "DejaVuSans.ttf"
@@ -62,7 +60,6 @@
 #define DETAIL_FONT "DejaVuSansMono.ttf"
 #define DETAIL_FONT_SIZE 9
 
-
 /* Screen dimensions */
 
 #define BORDER 12
@@ -96,26 +93,22 @@
 
 #define METER_WARNING_TIME 20 /* time in seconds for "red waveform" warning */
 
-
 /* Function key (F1-F12) definitions */
 
 #define FUNC_LOAD 0
 #define FUNC_RECUE 1
 #define FUNC_TIMECODE 2
 
-
 /* State variables used to trigger certain actions */
 
 #define UPDATE_NONE 0
 #define UPDATE_REDRAW 1
 
-
 /* Types of SDL_USEREVENT */
 
 #define EVENT_TICKER 0
 #define EVENT_QUIT 1
 
-
 /* Macro functions */
 
 #define MIN(x,y) ((x)<(y)?(x):(y))
@@ -126,19 +119,17 @@
 #define UPDATE(sf, rect) SDL_UpdateRect(sf, (rect)->x, (rect)->y, \
                                         (rect)->w, (rect)->h)
 
-
 /* List of directories to use as search path for fonts. */
 
 static const char *font_dirs[] = {
     "/usr/X11R6/lib/X11/fonts/TTF",
-    "/usr/share/fonts/truetype/ttf-dejavu/", 
+    "/usr/share/fonts/truetype/ttf-dejavu/",
     "/usr/share/fonts/ttf-dejavu",
     "/usr/share/fonts/dejavu",
     "/usr/share/fonts/TTF",
     NULL
 };
 
-
 static TTF_Font *clock_font, *deci_font, *detail_font, *font, *em_font;
 
 static SDL_Color background_col = {0, 0, 0, 255},
@@ -153,14 +144,16 @@ static SDL_Color background_col = {0, 0, 0, 255},
 
 static int spinner_angle[SPINNER_SIZE * SPINNER_SIZE];
 
-
 struct rect_t {
     signed short x, y, w, h;
 };
 
-
-/* Split the given rectangle split v pixels from the top, with a gap
- * of 'space' between the output rectangles */
+/*
+ * Split the given rectangle split v pixels from the top, with a gap
+ * of 'space' between the output rectangles
+ *
+ * Post: upper and lower are set
+ */
 
 static void split_top(const struct rect_t *source, struct rect_t *upper,
                       struct rect_t *lower, int v, int space)
@@ -184,8 +177,9 @@ static void split_top(const struct rect_t *source, struct rect_t *upper,
     }
 }
 
-
-/* As above, x pixels from the bottom */
+/*
+ * As above, x pixels from the bottom
+ */
 
 static void split_bottom(const struct rect_t *source, struct rect_t *upper,
                          struct rect_t *lower, int v, int space)
@@ -193,8 +187,9 @@ static void split_bottom(const struct rect_t *source, struct rect_t *upper,
     split_top(source, upper, lower, source->h - v - space, space);
 }
 
-
-/* As above, v pixels from the left */
+/*
+ * As above, v pixels from the left
+ */
 
 static void split_left(const struct rect_t *source, struct rect_t *left,
                        struct rect_t *right, int v, int space)
@@ -218,8 +213,9 @@ static void split_left(const struct rect_t *source, struct rect_t *left,
     }
 }
 
-
-/* As above, v pixels from the right */
+/*
+ * As above, v pixels from the right
+ */
 
 static void split_right(const struct rect_t *source, struct rect_t *left,
                         struct rect_t *right, int v, int space)
@@ -227,6 +223,9 @@ static void split_right(const struct rect_t *source, struct rect_t *left,
     split_left(source, left, right, source->w - v - space, space);
 }
 
+/*
+ * Convert the given time (in milliseconds) to displayable time
+ */
 
 static void time_to_clock(char *buf, char *deci, int t)
 {
@@ -241,7 +240,7 @@ static void time_to_clock(char *buf, char *deci, int t)
     minutes = (t / 60 / 1000) % (60*60);
     seconds = (t / 1000) % 60;
     frac = t % 1000;
-    
+
     if (neg)
         *buf++ = '-';
 
@@ -249,9 +248,10 @@ static void time_to_clock(char *buf, char *deci, int t)
     sprintf(deci, "%03d", frac);
 }
 
-
-/* Calculate a lookup which maps a position on screen to an angle,
- * relative to the centre of the spinner */
+/*
+ * Calculate a lookup which maps a position on screen to an angle,
+ * relative to the centre of the spinner
+ */
 
 static void calculate_spinner_lookup(int *angle, int *distance, int size)
 {
@@ -269,18 +269,18 @@ static void calculate_spinner_lookup(int *angle, int *distance, int size)
 
             else if (nc == 0) {
                 theta = 0;
-                
+
                 if (nr < 0)
                     theta = M_PI;
-            
+
             } else {
                 rat = (float)(nc) / -nr;
                 theta = atanf(rat);
-                
+
                 if (rat < 0)
                     theta += M_PI;
             }
-            
+
             if (nc <= 0)
                 theta += M_PI;
 
@@ -293,13 +293,18 @@ static void calculate_spinner_lookup(int *angle, int *distance, int size)
             if (distance)
                 distance[r * size + c] = sqrt(SQ(nc) + SQ(nr));
         }
-    }    
+    }
 }
 
-
-/* Open a font, given the leafname. This scans the available font
- * directories for the file, to account for different software
- * distributions. */
+/*
+ * Open a font, given the leafname
+ *
+ * This scans the available font directories for the file, to account
+ * for different software distributions.
+ *
+ * As this is an SDL (it is not an X11 app) we prefer to avoid the use
+ * of fontconfig to select fonts.
+ */
 
 static TTF_Font* open_font(const char *name, int size) {
     int r;
@@ -311,34 +316,34 @@ static TTF_Font* open_font(const char *name, int size) {
     dir = &font_dirs[0];
 
     while (*dir) {
-        
+
         sprintf(buf, "%s/%s", *dir, name);
-        
+
         r = stat(buf, &st);
 
         if (r != -1) { /* something exists at this path */
             fprintf(stderr, "Loading font '%s', %dpt...\n", buf, size);
-            
+
             font = TTF_OpenFont(buf, size);
             if (!font) {
                 fputs("Font error: ", stderr);
                 fputs(TTF_GetError(), stderr);
                 fputc('\n', stderr);
-            }            
+            }
             return font; /* or NULL */
         }
-        
+
         if (errno != ENOENT) {
             perror("stat");
             return NULL;
         }
-        
+
         dir++;
         continue;
     }
 
     fprintf(stderr, "Font '%s' cannot be found in", name);
-    
+
     dir = &font_dirs[0];
     while (*dir) {
         fputc(' ', stderr);
@@ -351,32 +356,38 @@ static TTF_Font* open_font(const char *name, int size) {
     return NULL;
 }
 
+/*
+ * Load all fonts
+ */
 
 static int load_fonts(void)
 {
     clock_font = open_font(CLOCK_FONT, CLOCK_FONT_SIZE);
     if (!clock_font)
         return -1;
-    
+
     deci_font = open_font(DECI_FONT, DECI_FONT_SIZE);
     if (!deci_font)
         return -1;
-    
+
     font = open_font(FONT, FONT_SIZE);
     if (!font)
         return -1;
-    
+
     em_font = open_font(EM_FONT, FONT_SIZE);
     if (!em_font)
         return -1;
-    
+
     detail_font = open_font(DETAIL_FONT, DETAIL_FONT_SIZE);
     if (!detail_font)
         return -1;
-    
+
     return 0;
 }
 
+/*
+ * Free resources associated with fonts
+ */
 
 static void clear_fonts(void)
 {
@@ -387,12 +398,16 @@ static void clear_fonts(void)
     TTF_CloseFont(detail_font);
 }
 
-
 static Uint32 palette(SDL_Surface *sf, SDL_Color *col)
 {
     return SDL_MapRGB(sf->format, col->r, col->g, col->b);
 }
 
+/*
+ * Draw text at the given coordinates
+ *
+ * Return: width of text drawn
+ */
 
 static int draw_font(SDL_Surface *sf, int x, int y, int w, int h,
                      const char *buf, TTF_Font *font,
@@ -411,18 +426,18 @@ static int draw_font(SDL_Surface *sf, int x, int y, int w, int h,
 
     } else {
         rendered = TTF_RenderText_Shaded(font, buf, fg, bg);
-        
+
         src.x = 0;
         src.y = 0;
         src.w = MIN(w, rendered->w);
         src.h = MIN(h, rendered->h);
-        
+
         dst.x = x;
         dst.y = y;
-        
+
         SDL_BlitSurface(rendered, &src, sf, &dst);
         SDL_FreeSurface(rendered);
-    } 
+    }
 
     /* Complete the remaining space with a blank rectangle */
 
@@ -430,7 +445,7 @@ static int draw_font(SDL_Surface *sf, int x, int y, int w, int h,
         fill.x = x + src.w;
         fill.y = y;
         fill.w = w - src.w;
-        fill.h = h;   
+        fill.h = h;
         SDL_FillRect(sf, &fill, palette(sf, &bg));
     }
 
@@ -445,6 +460,11 @@ static int draw_font(SDL_Surface *sf, int x, int y, int w, int h,
     return src.w;
 }
 
+/*
+ * Draw text at the given rectangle
+ *
+ * Return: width of text drawn
+ */
 
 static int draw_font_rect(SDL_Surface *surface, const struct rect_t *rect,
                           const char *buf, TTF_Font *font,
@@ -454,8 +474,9 @@ static int draw_font_rect(SDL_Surface *surface, const struct rect_t *rect,
                      buf, font, fg, bg);
 }
 
-
-/* Draw the display of artist name and track name */
+/*
+ * Draw the display of artist name and track name
+ */
 
 static void draw_track_summary(SDL_Surface *surface, const struct rect_t *rect,
                                struct track_t *track)
@@ -470,8 +491,9 @@ static void draw_track_summary(SDL_Surface *surface, const struct rect_t *rect,
                    em_font, text_col, background_col);
 }
 
-
-/* Draw a single clock, in hours:minutes.seconds format */
+/*
+ * Draw a single time in milliseconds in hours:minutes.seconds format
+ */
 
 static void draw_clock(SDL_Surface *surface, const struct rect_t *rect, int t,
                        SDL_Color col)
@@ -482,22 +504,22 @@ static void draw_clock(SDL_Surface *surface, const struct rect_t *rect, int t,
     struct rect_t sr;
 
     time_to_clock(hms, deci, t);
-    
+
     v = draw_font_rect(surface, rect, hms, clock_font, col, background_col);
-    
+
     offset = CLOCK_FONT_SIZE - DECI_FONT_SIZE * 1.04;
 
-    sr.x = rect->x + v; 
+    sr.x = rect->x + v;
     sr.y = rect->y + offset;
     sr.w = rect->w - v;
     sr.h = rect->h - offset;
-    
+
     draw_font_rect(surface, &sr, deci, deci_font, col, background_col);
 }
 
-
-/* Draw the visual display of the input audio to the timecoder (the
- * 'scope') */
+/*
+ * Draw the visual monitor of the input audio to the timecoder
+ */
 
 static void draw_scope(SDL_Surface *surface, const struct rect_t *rect,
                        struct timecoder_t *tc)
@@ -520,14 +542,17 @@ static void draw_scope(SDL_Surface *surface, const struct rect_t *rect,
 
             p[0] = v;
             p[1] = p[0];
-            p[2] = p[1];            
+            p[2] = p[1];
         }
-    }    
+    }
 }
 
-
-/* Draw the spinner which shows the rotational position of the record,
- * and matches the physical rotation of the vinyl record */
+/*
+ * Draw the spinner
+ *
+ * The spinner shows the rotational position of the record, and
+ * matches the physical rotation of the vinyl record.
+ */
 
 static void draw_spinner(SDL_Surface *surface, const struct rect_t *rect,
                          struct player_t *pl)
@@ -576,11 +601,12 @@ static void draw_spinner(SDL_Surface *surface, const struct rect_t *rect,
                 p[2] = col.r;
             }
         }
-    }    
+    }
 }
 
-
-/* Draw the clocks which show time elapsed and time remaining */
+/*
+ * Draw the clocks which show time elapsed and time remaining
+ */
 
 static void draw_deck_clocks(SDL_Surface *surface, const struct rect_t *rect,
                              struct player_t *pl)
@@ -620,9 +646,10 @@ static void draw_deck_clocks(SDL_Surface *surface, const struct rect_t *rect,
     draw_clock(surface, &lower, remain, col);
 }
 
-
-/* Draw the high-level overview meter which shows the whole length
- * of the track */
+/*
+ * Draw the high-level overview meter which shows the whole length
+ * of the track
+ */
 
 static void draw_overview(SDL_Surface *surface, const struct rect_t *rect,
                           struct track_t *tr, int position)
@@ -707,16 +734,17 @@ static void draw_overview(SDL_Surface *surface, const struct rect_t *rect,
     }
 }
 
-
-/* Draw the close-up meter, which can be zoomed to a level set by
- * 'scale' */
+/*
+ * Draw the close-up meter, which can be zoomed to a level set by
+ * 'scale'
+ */
 
 static void draw_closeup(SDL_Surface *surface, const struct rect_t *rect,
                          struct track_t *tr, int position, int scale)
 {
-    int x, y, w, h, r, c, sp, fade, bytes_per_pixel, pitch, height;
-    Uint8 *pixels, *p;
-    SDL_Color col;
+    int x, y, w, h, c;
+    size_t bytes_per_pixel, pitch;
+    Uint8 *pixels;
 
     x = rect->x;
     y = rect->y;
@@ -727,7 +755,13 @@ static void draw_closeup(SDL_Surface *surface, const struct rect_t *rect,
     bytes_per_pixel = surface->format->BytesPerPixel;
     pitch = surface->pitch;
 
+    /* Draw in columns. This may seem like a performance hit,
+     * but oprofile shows it makes no difference */
+
     for (c = 0; c < w; c++) {
+        int r, sp, height, fade;
+        Uint8 *p;
+        SDL_Color col;
 
         /* Work out the meter height in pixels for this column */
 
@@ -772,8 +806,9 @@ static void draw_closeup(SDL_Surface *surface, const struct rect_t *rect,
     }
 }
 
-
-/* Draw the audio meters for a deck */                         
+/*
+ * Draw the audio meters for a deck
+ */
 
 static void draw_meters(SDL_Surface *surface, const struct rect_t *rect,
                         struct track_t *tr, int position, int scale)
@@ -790,17 +825,18 @@ static void draw_meters(SDL_Surface *surface, const struct rect_t *rect,
     draw_closeup(surface, &closeup, tr, position, scale);
 }
 
-
-/* Draw the current playback status -- clocks, spinner and scope */
+/*
+ * Draw the current playback status -- clocks, spinner and scope
+ */
 
 static void draw_deck_top(SDL_Surface *surface, const struct rect_t *rect,
                           struct player_t *pl)
 {
     struct rect_t clocks, left, right, spinner, scope;
-    
+
     split_left(rect, &clocks, &right, CLOCKS_WIDTH, SPACER);
 
-    /* If there is no timecoder to display information on, or not enough 
+    /* If there is no timecoder to display information on, or not enough
      * available space, just draw clocks which span the overall space */
 
     if (!pl->timecode_control || right.w < 0) {
@@ -808,7 +844,7 @@ static void draw_deck_top(SDL_Surface *surface, const struct rect_t *rect,
         return;
     }
 
-    draw_deck_clocks(surface, &clocks, pl);    
+    draw_deck_clocks(surface, &clocks, pl);
 
     split_right(&right, &left, &spinner, SPINNER_SIZE, SPACER);
     if (left.w < 0)
@@ -823,42 +859,52 @@ static void draw_deck_top(SDL_Surface *surface, const struct rect_t *rect,
     draw_scope(surface, &scope, pl->timecoder);
 }
 
-
-/* Draw the textual description of playback status, which includes
- * information on the timecode */
+/*
+ * Draw the textual description of playback status, which includes
+ * information on the timecode
+ */
 
 static void draw_deck_status(SDL_Surface *surface,
                                  const struct rect_t *rect,
                                  struct player_t *pl)
 {
-    char buf[128];
+    char buf[128], *c;
     int tc;
 
+    c = buf;
+
+    c += sprintf(c, "%s: ", pl->timecoder->def->name);
+
     tc = timecoder_get_position(pl->timecoder, NULL);
-    if (pl->timecode_control && tc != -1)
-        sprintf(buf, "timecode:%d      ", tc);
-    else
-        sprintf(buf, "timecode:        ");
-    
-    sprintf(buf + 17, "pitch:%+0.2f (sync %0.2f %+.5fs = %+0.2f)  %s",
+    if (pl->timecode_control && tc != -1) {
+        c += sprintf(c, "%7d ", tc);
+    } else {
+        c += sprintf(c, "        ");
+    }
+
+    sprintf(c, "pitch:%+0.2f (sync %0.2f %+.5fs = %+0.2f)  %s",
             pl->pitch,
             pl->sync_pitch,
             pl->last_difference,
             pl->pitch * pl->sync_pitch,
             pl->recalibrate ? "RCAL  " : "");
-    
+
     draw_font_rect(surface, rect, buf, detail_font,
                    detail_col, background_col);
 }
 
-
-/* Draw a single deck */
+/*
+ * Draw a single deck
+ */
 
 static void draw_deck(SDL_Surface *surface, const struct rect_t *rect,
-                      struct player_t *pl, int meter_scale)
+                      struct deck_t *deck, int meter_scale)
 {
     int position;
     struct rect_t track, top, meters, status, rest, lower;
+    struct player_t *pl;
+
+    pl = &deck->player;
 
     position = (pl->position - pl->offset) * pl->track->rate;
 
@@ -873,8 +919,8 @@ static void draw_deck(SDL_Surface *surface, const struct rect_t *rect,
         lower = rest;
     else
         draw_deck_top(surface, &top, pl);
-    
-    split_bottom(&lower, &meters, &status, FONT_SPACE, SPACER); 
+
+    split_bottom(&lower, &meters, &status, FONT_SPACE, SPACER);
     if (meters.h < 64)
         meters = lower;
     else
@@ -883,12 +929,12 @@ static void draw_deck(SDL_Surface *surface, const struct rect_t *rect,
     draw_meters(surface, &meters, pl->track, position, meter_scale);
 }
 
-
-/* Draw all the decks in the system */
+/*
+ * Draw all the decks in the system left to right
+ */
 
 static void draw_decks(SDL_Surface *surface, const struct rect_t *rect,
-                       int ndecks, struct player_t **player,
-                       int meter_scale)
+                       struct deck_t deck[], size_t ndecks, int meter_scale)
 {
     int d, deck_width;
     struct rect_t single;
@@ -900,12 +946,13 @@ static void draw_decks(SDL_Surface *surface, const struct rect_t *rect,
 
     for (d = 0; d < ndecks; d++) {
         single.x = rect->x + (deck_width + BORDER) * d;
-        draw_deck(surface, &single, player[d], meter_scale);
+        draw_deck(surface, &single, &deck[d], meter_scale);
     }
 }
 
-
-/* Draw the status bar */
+/*
+ * Draw the status bar
+ */
 
 static void draw_status(SDL_Surface *sf, const struct rect_t *rect,
                         const char *text)
@@ -913,8 +960,9 @@ static void draw_status(SDL_Surface *sf, const struct rect_t *rect,
     draw_font_rect(sf, rect, text, detail_font, detail_col, background_col);
 }
 
-
-/* Draw the search field which the user types into */
+/*
+ * Draw the search field which the user types into
+ */
 
 static void draw_search(SDL_Surface *surface, const struct rect_t *rect,
                         struct selector_t *sel)
@@ -954,9 +1002,10 @@ static void draw_search(SDL_Surface *surface, const struct rect_t *rect,
     draw_font_rect(surface, &rtext, cm, em_font, detail_col, background_col);
 }
 
-
-/* Draw a vertical scroll bar representing our view on a list of the
- * given number of entries */
+/*
+ * Draw a vertical scroll bar representing our view on a list of the
+ * given number of entries
+ */
 
 static void draw_scroll_bar(SDL_Surface *surface, const struct rect_t *rect,
                             struct scroll_t *scroll)
@@ -984,9 +1033,11 @@ static void draw_scroll_bar(SDL_Surface *surface, const struct rect_t *rect,
     }
 }
 
-
-/* Display a crate listing, with scrollbar and current
- * selection. Return the number of lines which fit on the display. */
+/*
+ * Display a crate listing, with scrollbar and current selection
+ *
+ * Return: the number of lines which fit on the display.
+ */
 
 static void draw_crates(SDL_Surface *surface, const struct rect_t *rect,
                         struct selector_t *sel)
@@ -1043,9 +1094,12 @@ static void draw_crates(SDL_Surface *surface, const struct rect_t *rect,
     draw_scroll_bar(surface, &rs, &sel->crates);
 }
 
-
-/* Display a record library listing, with scrollbar and current
- * selection. Return the number of lines which fit on the display. */
+/*
+ * Display a record library listing, with scrollbar and current
+ * selection
+ *
+ * Return: the number of lines which fit on the display
+ */
 
 static void draw_records(SDL_Surface *surface, const struct rect_t *rect,
                          struct selector_t *sel)
@@ -1055,7 +1109,7 @@ static void draw_records(SDL_Surface *surface, const struct rect_t *rect,
     struct record_t *re;
     SDL_Rect box;
     SDL_Color col;
-    
+
     x = rect->x;
     y = rect->y;
     w = rect->w;
@@ -1068,12 +1122,12 @@ static void draw_records(SDL_Surface *surface, const struct rect_t *rect,
 
     for (n = 0; n + sel->records.offset < sel->view_listing->entries; n++) {
         re = sel->view_listing->record[n + sel->records.offset];
-    
+
         if ((n + 1) * FONT_SPACE > h)
             break;
 
         r = y + n * FONT_SPACE;
-    
+
         if (n + sel->records.offset == sel->records.selected)
             col = selected_col;
         else
@@ -1088,21 +1142,21 @@ static void draw_records(SDL_Surface *surface, const struct rect_t *rect,
         box.h = FONT_SPACE;
 
         SDL_FillRect(surface, &box, palette(surface, &col));
-            
+
         draw_font(surface, x + RESULTS_ARTIST_WIDTH + SPACER, r,
                   w - RESULTS_ARTIST_WIDTH - SPACER, FONT_SPACE,
                   re->title, em_font, text_col, col);
     }
 
     /* Blank any remaining space */
-    
+
     box.x = x;
     box.y = y + n * FONT_SPACE;
     box.w = w;
     box.h = h - (n * FONT_SPACE);
-    
+
     SDL_FillRect(surface, &box, palette(surface, &background_col));
-    
+
     rs.x = ox;
     rs.y = y;
     rs.w = SCROLLBAR_SIZE;
@@ -1110,9 +1164,10 @@ static void draw_records(SDL_Surface *surface, const struct rect_t *rect,
     draw_scroll_bar(surface, &rs, &sel->records);
 }
 
-
-/* Display the music library, which consists of the query, and search
- * results */
+/*
+ * Display the music library, which consists of the query, and search
+ * results
+ */
 
 static void draw_library(SDL_Surface *surface, const struct rect_t *rect,
                          struct selector_t *sel)
@@ -1135,9 +1190,10 @@ static void draw_library(SDL_Surface *surface, const struct rect_t *rect,
     }
 }
 
-
-/* Initiate the process of loading a record from the library into a
- * track in memory */
+/*
+ * Initiate the process of loading a record from the library into a
+ * track in memory
+ */
 
 static void do_loading(struct interface_t *interface,
                        struct track_t *track, struct record_t *record)
@@ -1153,16 +1209,15 @@ static void do_loading(struct interface_t *interface,
     (void)rig_awaken(interface->rig);
 }
 
-
-/* Handle a single key event. Return true if the selector needs
- * to be redrawn */
+/*
+ * Handle a single key event
+ *
+ * Return: true if the selector needs to be redrawn, otherwise false
+ */
 
 static bool handle_key(struct interface_t *in, struct selector_t *sel,
                        int *meter_scale, SDLKey key, SDLMod mod)
 {
-    struct player_t *pl;
-    struct record_t* re;
-
     if (key >= SDLK_a && key <= SDLK_z) {
         selector_search_refine(sel, (key - SDLK_a) + 'a');
         return true;
@@ -1236,27 +1291,35 @@ static bool handle_key(struct interface_t *in, struct selector_t *sel,
         fprintf(stderr, "Meter scale increased to %d\n", *meter_scale);
 
     } else if (key >= SDLK_F1 && key <= SDLK_F12) {
-        int func, deck;
+        size_t d;
 
         /* Handle the function key press in groups of four --
 	 * F1-F4 (deck 0), F5-F8 (deck 1) etc. */
 
-        func = (key - SDLK_F1) % 4;
-        deck = (key - SDLK_F1) / 4;
+        d = (key - SDLK_F1) / 4;
 
-        if (deck < in->players) {
-            pl = in->player[deck];
+        if (d < in->ndeck) {
+            int func;
+            struct deck_t *deck;
+            struct player_t *pl;
+            struct record_t *re;
+            struct timecoder_t *tc;
+
+            func = (key - SDLK_F1) % 4;
+
+            deck = &in->deck[d];
+            pl = &deck->player;
+            tc = &deck->timecoder;
 
             if (mod & KMOD_SHIFT) {
-                if (func < in->timecoders)
-                    player_connect_timecoder(pl, in->timecoder[func]);
+                if (func < in->ndeck)
+                    player_set_timecoder(pl, &in->deck[func].timecoder);
 
             } else switch(func) {
-
             case FUNC_LOAD:
                 re = selector_current(sel);
                 if (re != NULL)
-                    do_loading(in, pl->track, re);
+                    do_loading(in, &deck->track, re);
                 break;
 
             case FUNC_RECUE:
@@ -1264,7 +1327,11 @@ static bool handle_key(struct interface_t *in, struct selector_t *sel,
                 break;
 
             case FUNC_TIMECODE:
-                (void)player_toggle_timecode_control(pl);
+                if (mod & KMOD_CTRL) {
+                    timecoder_cycle_definition(tc);
+                } else {
+                    (void)player_toggle_timecode_control(pl);
+                }
                 break;
             }
         }
@@ -1273,6 +1340,9 @@ static bool handle_key(struct interface_t *in, struct selector_t *sel,
     return false;
 }
 
+/*
+ * Action on size change event on the main window
+ */
 
 static SDL_Surface* set_size(int w, int h, struct rect_t *rect)
 {
@@ -1283,33 +1353,35 @@ static SDL_Surface* set_size(int w, int h, struct rect_t *rect)
         fprintf(stderr, "%s\n", SDL_GetError());
         return NULL;
     }
-    
+
     rect->x = BORDER;
     rect->y = BORDER;
     rect->w = w - 2 * BORDER;
     rect->h = h - 2 * BORDER;
 
     fprintf(stderr, "New interface size is %dx%d.\n", w, h);
-    
+
     return surface;
 }
 
+/*
+ * Timer which posts a screen redraw event
+ */
 
 static Uint32 ticker(Uint32 interval, void *p)
 {
     SDL_Event event;
-    
+
     if (!SDL_PeepEvents(&event, 1, SDL_PEEKEVENT, SDL_EVENTMASK(SDL_USEREVENT)))
     {
         event.type = SDL_USEREVENT;
         event.user.code = EVENT_TICKER;
         SDL_PushEvent(&event);
-    }        
+    }
 
     return interval;
 }
 
-
 /*
  * The SDL interface thread
  */
@@ -1352,22 +1424,16 @@ static int interface_main(struct interface_t *in)
             if (!surface)
                 return -1;
 
-            if (library_update < UPDATE_REDRAW)
-                library_update = UPDATE_REDRAW;
-
-            if (decks_update < UPDATE_REDRAW)
-                decks_update = UPDATE_REDRAW;
-
-            if (status_update < UPDATE_REDRAW)
-                status_update = UPDATE_REDRAW;
+            library_update = UPDATE_REDRAW;
+            decks_update = UPDATE_REDRAW;
+            status_update = UPDATE_REDRAW;
 
             break;
-            
+
         case SDL_USEREVENT:
             switch (event.user.code) {
             case EVENT_TICKER: /* request to poll the clocks */
-                if (decks_update < UPDATE_REDRAW)
-                    decks_update = UPDATE_REDRAW;
+                decks_update = UPDATE_REDRAW;
                 break;
 
             case EVENT_QUIT: /* internal request to finish this thread */
@@ -1396,7 +1462,7 @@ static int interface_main(struct interface_t *in)
             status_update = UPDATE_NONE;
         }
 
-        split_top(&rtmp, &rplayers, &rlibrary, PLAYER_HEIGHT, SPACER);        
+        split_top(&rtmp, &rplayers, &rlibrary, PLAYER_HEIGHT, SPACER);
         if (rlibrary.h < LIBRARY_MIN_HEIGHT || rlibrary.w < LIBRARY_MIN_WIDTH) {
             rplayers = rtmp;
             library_update = UPDATE_NONE;
@@ -1408,14 +1474,14 @@ static int interface_main(struct interface_t *in)
         /* If there's been a change to the library search results,
          * check them over and display them. */
 
-        if (library_update >= UPDATE_REDRAW) {
+        if (library_update == UPDATE_REDRAW) {
 
             if (selector_current(&in->selector) != NULL)
                 status = selector_current(&in->selector)->pathname;
             else
                 status = "No search results found";
             status_update = UPDATE_REDRAW;
-          
+
             LOCK(surface);
             draw_library(surface, &rlibrary, &in->selector);
             UNLOCK(surface);
@@ -1425,7 +1491,7 @@ static int interface_main(struct interface_t *in)
 
         /* If there's been a change to status, redisplay it */
 
-        if (status_update >= UPDATE_REDRAW) {
+        if (status_update == UPDATE_REDRAW) {
             LOCK(surface);
             draw_status(surface, &rstatus, status);
             UNLOCK(surface);
@@ -1436,10 +1502,9 @@ static int interface_main(struct interface_t *in)
         /* If it's due, redraw the players. This is triggered by the
          * timer event. */
 
-        if (decks_update >= UPDATE_REDRAW) {
+        if (decks_update == UPDATE_REDRAW) {
             LOCK(surface);
-            draw_decks(surface, &rplayers, in->players, in->player,
-                       meter_scale);
+            draw_decks(surface, &rplayers, in->deck, in->ndeck, meter_scale);
             UNLOCK(surface);
             UPDATE(surface, &rplayers);
             decks_update = UPDATE_NONE;
@@ -1466,23 +1531,18 @@ static void* launch(void *p)
  * error
  */
 
-int interface_start(struct interface_t *in, size_t ndeck,
-                    struct player_t *pl, struct timecoder_t *tc,
+int interface_start(struct interface_t *in, struct deck_t deck[], size_t ndeck,
                     struct library_t *lib, struct rig_t *rig)
 {
     size_t n;
 
-    assert(ndeck <= MAX_DECKS);
-
     for (n = 0; n < ndeck; n++) {
-        in->player[n] = &pl[n];
-        in->timecoder[n] = &tc[n];
-        if (timecoder_monitor_init(in->timecoder[n], SCOPE_SIZE) == -1)
+        if (timecoder_monitor_init(&deck[n].timecoder, SCOPE_SIZE) == -1)
             return -1;
     }
 
-    in->players = ndeck;
-    in->timecoders = ndeck;
+    in->deck = deck;
+    in->ndeck = ndeck;
 
     in->rig = rig;
     selector_init(&in->selector, lib);
@@ -1534,8 +1594,8 @@ void interface_stop(struct interface_t *in)
     if (pthread_join(in->ph, NULL) != 0)
         abort();
 
-    for (n = 0; n < in->timecoders; n++)
-        timecoder_monitor_clear(in->timecoder[n]);
+    for (n = 0; n < in->ndeck; n++)
+        timecoder_monitor_clear(&in->deck[n].timecoder);
 
     selector_clear(&in->selector);
 
