@@ -27,8 +27,6 @@
 #include <sys/soundcard.h>
 
 #include "oss.h"
-#include "timecoder.h"
-#include "player.h"
 
 #define FRAME 32 /* maximum read size */
 
@@ -40,7 +38,7 @@ struct oss_t {
 };
 
 
-static void clear(struct device_t *dv)
+static void clear(struct device *dv)
 {
     int r;
     struct oss_t *oss = (struct oss_t*)dv->local;
@@ -99,7 +97,7 @@ static int pull(int fd, signed short *pcm, int samples)
 }
 
 
-static int handle(struct device_t *dv)
+static int handle(struct device *dv)
 {
     signed short pcm[FRAME * DEVICE_CHANNELS];
     int samples;
@@ -111,27 +109,14 @@ static int handle(struct device_t *dv)
         samples = pull(oss->fd, pcm, FRAME);
         if (samples == -1)
             return -1;
-        
-        if (dv->timecoder)
-            timecoder_submit(dv->timecoder, pcm, samples);
+        device_submit(dv, pcm, samples);
     }
 
     /* Check the output buffer for playback */
     
     if (oss->pe->revents & POLLOUT) {
-
-        /* Always push some audio to the soundcard, even if it means
-         * silence. This has shown itself to be much more reliable
-         * than starting and stopping -- which can affect other
-         * devices in the system. */
-        
-        if (dv->player)
-            player_collect(dv->player, pcm, FRAME, oss->rate);
-        else
-            memset(pcm, 0, FRAME * DEVICE_CHANNELS * sizeof(short));
-        
+        device_collect(dv, pcm, FRAME);
         samples = push(oss->fd, pcm, FRAME);
-        
         if (samples == -1)
             return -1;
     }
@@ -140,7 +125,7 @@ static int handle(struct device_t *dv)
 }
 
 
-static ssize_t pollfds(struct device_t *dv, struct pollfd *pe, size_t z)
+static ssize_t pollfds(struct device *dv, struct pollfd *pe, size_t z)
 {
     struct oss_t *oss = (struct oss_t*)dv->local;
 
@@ -155,7 +140,7 @@ static ssize_t pollfds(struct device_t *dv, struct pollfd *pe, size_t z)
 }
 
 
-static unsigned int sample_rate(struct device_t *dv)
+static unsigned int sample_rate(struct device *dv)
 {
     struct oss_t *oss = (struct oss_t*)dv->local;
 
@@ -163,17 +148,15 @@ static unsigned int sample_rate(struct device_t *dv)
 }
 
 
-static struct device_type_t oss_type = {
+static struct device_ops oss_ops = {
     .pollfds = pollfds,
     .handle = handle,
     .sample_rate = sample_rate,
-    .start = NULL,
-    .stop = NULL,
     .clear = clear
 };
 
 
-int oss_init(struct device_t *dv, const char *filename, unsigned int rate,
+int oss_init(struct device *dv, const char *filename, unsigned int rate,
 	     unsigned short buffers, unsigned short fragment)
 {
     int p, fd;
@@ -227,19 +210,18 @@ int oss_init(struct device_t *dv, const char *filename, unsigned int rate,
         return -1;
     }
 
-    dv->local = malloc(sizeof(struct oss_t));
-    if (!dv->local) {
+    oss = malloc(sizeof(struct oss_t));
+    if (oss == NULL) {
         perror("malloc");
         goto fail;
     }
 
-    oss = (struct oss_t*)dv->local;
-
     oss->fd = fd;
     oss->pe = NULL;
     oss->rate = rate;
 
-    dv->type = &oss_type;
+    dv->local = oss;
+    dv->ops = &oss_ops;
 
     return 0;
 
