@@ -53,9 +53,9 @@
  * Return: the cubic interpolation of the sample at position 2 + mu
  */
 
-static inline float cubic_interpolate(float y[4], float mu)
+static inline double cubic_interpolate(double y[4], double mu)
 {
-    float a0, a1, a2, a3, mu2;
+    double a0, a1, a2, a3, mu2;
 
     mu2 = SQ(mu);
     a0 = y[3] - y[2] - y[0] + y[1];
@@ -76,23 +76,22 @@ static inline float cubic_interpolate(float y[4], float mu)
  * Post: buffer at pcm is filled with the given number of samples
  */
 
-static double build_pcm(signed short *pcm, int samples, int rate,
-                        struct track_t *tr, double position, float pitch,
-                        float start_vol, float end_vol)
+static double build_pcm(signed short *pcm, unsigned samples, double sample_dt,
+                        struct track_t *tr, double position, double pitch,
+                        double start_vol, double end_vol)
 {
     int s;
-    double sample, step;
-    float vol, gradient;
+    double sample, step, vol, gradient;
 
     sample = position * tr->rate;
-    step = (double)pitch * tr->rate / rate;
+    step = sample_dt * pitch * tr->rate;
 
     vol = start_vol;
     gradient = (end_vol - start_vol) / samples;
 
     for (s = 0; s < samples; s++) {
         int c, sa, q;
-        float f, i[PLAYER_CHANNELS][4];
+        double f, i[PLAYER_CHANNELS][4];
 
         /* 4-sample window for interpolation */
 
@@ -112,15 +111,15 @@ static double build_pcm(signed short *pcm, int samples, int rate,
 
                 ts = track_get_sample(tr, sa);
                 for (c = 0; c < PLAYER_CHANNELS; c++)
-                    i[c][q] = (float)ts[c];
+                    i[c][q] = (double)ts[c];
             }
         }
 
         for (c = 0; c < PLAYER_CHANNELS; c++) {
-            float v;
+            double v;
 
             v = vol * cubic_interpolate(i[c], f)
-                + (float)(rand() % 32768) / 32768 - 0.5; /* dither */
+                + (double)(rand() % 32768) / 32768 - 0.5; /* dither */
 
             if (v > SHRT_MAX) {
                 *pcm++ = SHRT_MAX;
@@ -135,17 +134,33 @@ static double build_pcm(signed short *pcm, int samples, int rate,
         vol += gradient;
     }
 
-    return (double)pitch * samples / rate;
+    return sample_dt * pitch * samples;
+}
+
+/*
+ * Change the timecoder used by this playback
+ */
+
+void player_set_timecoder(struct player_t *pl, struct timecoder_t *tc)
+{
+    assert(tc != NULL);
+    pl->timecoder = tc;
+    pl->recalibrate = true;
+    pl->timecode_control = true;
 }
 
 /*
  * Post: player is initialised
  */
 
-void player_init(struct player_t *pl, struct track_t *track)
+void player_init(struct player_t *pl, unsigned int sample_rate,
+                 struct track_t *track, struct timecoder_t *tc)
 {
     assert(track != NULL);
+    assert(sample_rate != 0);
+    pl->sample_dt = 1.0 / sample_rate;
     pl->track = track;
+    player_set_timecoder(pl, tc);
 
     pl->position = 0.0;
     pl->offset = 0.0;
@@ -155,10 +170,6 @@ void player_init(struct player_t *pl, struct track_t *track)
     pl->pitch = 0.0;
     pl->sync_pitch = 1.0;
     pl->volume = 0.0;
-
-    pl->timecoder = NULL;
-    pl->timecode_control = false;
-    pl->recalibrate = false;
 }
 
 /*
@@ -170,17 +181,6 @@ void player_clear(struct player_t *pl)
 {
 }
 
-/*
- * Connect the given timecoder and enable timecode control
- */
-
-void player_connect_timecoder(struct player_t *pl, struct timecoder_t *tc)
-{
-    pl->timecoder = tc;
-    pl->recalibrate = true;
-    pl->timecode_control = true;
-}
-
 /*
  * Enable or disable timecode control
  */
@@ -214,8 +214,7 @@ bool player_toggle_timecode_control(struct player_t *pl)
 
 static int sync_to_timecode(struct player_t *pl)
 {
-    float when;
-    double tcpos;
+    double when, tcpos;
     signed int timecode;
 
     timecode = timecoder_get_position(pl->timecoder, &when);
@@ -260,10 +259,9 @@ static void calibrate_to_timecode_position(struct player_t *pl)
  * Cue to the zero position of the track
  */
 
-int player_recue(struct player_t *pl)
+void player_recue(struct player_t *pl)
 {
     pl->offset = pl->position;
-    return 0;
 }
 
 /*
@@ -276,13 +274,12 @@ int player_recue(struct player_t *pl)
  * Post: buffer at pcm is filled with the given number of samples
  */
 
-void player_collect(struct player_t *pl, signed short *pcm,
-                    int samples, int rate)
+void player_collect(struct player_t *pl, signed short *pcm, unsigned samples)
 {
     double diff;
     float dt, target_volume;
 
-    dt = (float)samples / rate;
+    dt = pl->sample_dt * samples;
 
     if (pl->timecode_control) {
         if (sync_to_timecode(pl) == -1)
@@ -335,7 +332,7 @@ void player_collect(struct player_t *pl, signed short *pcm,
 
     /* Sync pitch is applied post-filtering */
 
-    pl->position += build_pcm(pcm, samples, rate,
+    pl->position += build_pcm(pcm, samples, pl->sample_dt,
 			      pl->track,
                               pl->position - pl->offset,
                               pl->pitch * pl->sync_pitch,
