@@ -27,6 +27,7 @@
 #define PLAYER_CHANNELS 2
 
 struct player_t {
+    double sample_dt;
     struct track_t *track;
 
     /* Current playback parameters */
@@ -34,8 +35,8 @@ struct player_t {
     double position, /* seconds */
         target_position, /* seconds */
         offset, /* track start point in timecode */
-        last_difference; /* last known position minus target_position */
-    float pitch, /* from timecoder */
+        last_difference, /* last known position minus target_position */
+        pitch, /* from timecoder */
         sync_pitch, /* pitch required to sync to timecode signal */
         volume;
 
@@ -48,16 +49,16 @@ struct player_t {
         recalibrate; /* re-sync offset at next opportunity */
 };
 
-void player_init(struct player_t *pl, struct track_t *track);
+void player_init(struct player_t *pl, unsigned int sample_rate,
+                 struct track_t *track, struct timecoder_t *timecoder);
 void player_clear(struct player_t *pl);
 
-void player_connect_timecoder(struct player_t *pl, struct timecoder_t *tc);
+void player_set_timecoder(struct player_t *pl, struct timecoder_t *tc);
 void player_set_timecode_control(struct player_t *pl, bool on);
 bool player_toggle_timecode_control(struct player_t *pl);
 
-int player_recue(struct player_t *pl);
+void player_recue(struct player_t *pl);
 
-void player_collect(struct player_t *pl, signed short *pcm,
-                    int samples, int rate);
+void player_collect(struct player_t *pl, signed short *pcm, unsigned samples);
 
 #endif
