@@ -20,11 +20,11 @@
 #ifndef REALTIME_H
 #define REALTIME_H
 
+#include <poll.h>
+#include <pthread.h>
+#include <semaphore.h>
 #include <stdbool.h>
 
-#define MAX_DEVICES 4
-#define MAX_DEVICE_POLLFDS 32
-
 /*
  * State data for the realtime thread, maintained during rt_start and
  * rt_stop
@@ -32,19 +32,20 @@
 
 struct rt_t {
     pthread_t ph;
+    sem_t sem;
     bool finished;
 
     size_t ndv;
-    struct device_t *dv[MAX_DEVICES];
+    struct device *dv[3];
 
     size_t npt;
-    struct pollfd pt[MAX_DEVICE_POLLFDS];
+    struct pollfd pt[32];
 };
 
 void rt_init(struct rt_t *rt);
 void rt_clear(struct rt_t *rt);
 
-int rt_add_device(struct rt_t *rt, struct device_t *dv);
+int rt_add_device(struct rt_t *rt, struct device *dv);
 
 int rt_start(struct rt_t *rt);
 void rt_stop(struct rt_t *rt);
