@@ -1,15 +1,15 @@
-/* 
+/*
  * Copyright (C) 2011 Mark Hills <mark@pogo.org.uk>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * version 2, as published by the Free Software Foundation.
- * 
+ *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  * General Public License version 2 for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * version 2 along with this program; if not, write to the Free
  * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
@@ -22,16 +22,13 @@
 
 #include <stdbool.h>
 
-#include "device.h"
 #include "lut.h"
 #include "pitch.h"
 
-#define TIMECODER_CHANNELS DEVICE_CHANNELS
-
+#define TIMECODER_CHANNELS 2
 
 typedef unsigned int bits_t;
 
-
 struct timecode_def_t {
     char *name, *desc;
     int bits, /* number of bits in string */
@@ -45,7 +42,6 @@ struct timecode_def_t {
     struct lut_t lut;
 };
 
-
 struct timecoder_channel_t {
     int positive, /* wave is in positive part of cycle */
 	swapped; /* wave recently swapped polarity */
@@ -53,18 +49,17 @@ struct timecoder_channel_t {
     unsigned int crossing_ticker; /* samples since we last crossed zero */
 };
 
-
 struct timecoder_t {
     struct timecode_def_t *def;
     double speed;
 
     /* Precomputed values */
 
-    float dt, zero_alpha;
+    double dt, zero_alpha;
 
     /* Pitch information */
 
-    int forwards;
+    bool forwards;
     struct timecoder_channel_t primary, secondary;
     struct pitch_t pitch;
 
@@ -82,50 +77,63 @@ struct timecoder_t {
     int mon_size, mon_counter;
 };
 
-
+struct timecode_def_t* timecoder_find_definition(const char *name);
 void timecoder_free_lookup(void);
 
-int timecoder_init(struct timecoder_t *tc, const char *def_name, double speed,
-                   unsigned int sample_rate);
+void timecoder_init(struct timecoder_t *tc, struct timecode_def_t *def,
+                    double speed, unsigned int sample_rate);
 void timecoder_clear(struct timecoder_t *tc);
 
 int timecoder_monitor_init(struct timecoder_t *tc, int size);
 void timecoder_monitor_clear(struct timecoder_t *tc);
 
+void timecoder_cycle_definition(struct timecoder_t *tc);
 void timecoder_submit(struct timecoder_t *tc, signed short *pcm, size_t npcm);
+signed int timecoder_get_position(struct timecoder_t *tc, double *when);
 
-signed int timecoder_get_position(struct timecoder_t *tc, float *when);
+/*
+ * The timecode definition currently in use by this decoder
+ */
 
+static inline struct timecode_def_t* timecoder_get_definition(struct timecoder_t *tc)
+{
+    return tc->def;
+}
 
-/* Return the pitch relative to reference playback speed */
+/*
+ * Return the pitch relative to reference playback speed
+ */
 
-static inline float timecoder_get_pitch(struct timecoder_t *tc)
+static inline double timecoder_get_pitch(struct timecoder_t *tc)
 {
     return pitch_current(&tc->pitch) / tc->speed;
 }
 
-
-/* The last 'safe' timecode value on the record. Beyond this value, we
+/*
+ * The last 'safe' timecode value on the record. Beyond this value, we
  * probably want to ignore the timecode values, as we will hit the
- * label of the record. */
+ * label of the record.
+ */
 
 static inline unsigned int timecoder_get_safe(struct timecoder_t *tc)
 {
     return tc->def->safe;
 }
 
-
-/* The resolution of the timecode. This is the number of bits per
- * second at reference playback speed */
+/*
+ * The resolution of the timecode. This is the number of bits per
+ * second at reference playback speed
+ */
 
 static inline double timecoder_get_resolution(struct timecoder_t *tc)
 {
     return tc->def->resolution * tc->speed;
 }
 
-
-/* The number of revolutions per second of the timecode vinyl,
- * used only for visual display */
+/*
+ * The number of revolutions per second of the timecode vinyl,
+ * used only for visual display
+ */
 
 static inline double timecoder_revs_per_sec(struct timecoder_t *tc)
 {
