@@ -17,6 +17,7 @@
  *
  */
 
+#include <assert.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -25,20 +26,18 @@
 #include <SDL.h> /* may override main() */
 
 #include "alsa.h"
+#include "deck.h"
 #include "device.h"
 #include "interface.h"
 #include "jack.h"
 #include "library.h"
 #include "oss.h"
-#include "player.h"
 #include "realtime.h"
 #include "rig.h"
 #include "timecoder.h"
 #include "track.h"
 #include "xwax.h"
 
-#define MAX_DECKS 3
-
 #define DEFAULT_OSS_BUFFERS 8
 #define DEFAULT_OSS_FRAGMENT 7
 
@@ -89,9 +88,9 @@ static void usage(FILE *fd)
 #endif
 
     fprintf(fd,
-      "Device options, -t, RPM options and -i apply to subsequent devices.\n"
-      "Option -s applies to subsequent directories.\n"
-      "Decks and audio directories can be specified multiple times.\n\n"
+      "The ordering of options is important. Options apply to subsequent\n"
+      "music libraries or decks, which can be given multiple times. See the\n"
+      "manual for details.\n\n"
       "Available timecodes (for use with -t):\n"
       "  serato_2a (default), serato_2b, serato_cd,\n"
       "  traktor_a, traktor_b, mixvibes_v2, mixvibes_7inch\n\n"
@@ -100,20 +99,30 @@ static void usage(FILE *fd)
 
 int main(int argc, char *argv[])
 {
-    int r, n, decks, oss_fragment, oss_buffers, rate, alsa_buffer;
-    char *endptr, *timecode, *importer, *scanner;
+    int r, n, decks;
+    char *importer, *scanner;
     double speed;
+    struct timecode_def_t *timecode;
 
-    struct device_t device[MAX_DECKS];
-    struct track_t track[MAX_DECKS];
-    struct player_t player[MAX_DECKS];
-    struct timecoder_t timecoder[MAX_DECKS];
-
+    struct deck_t deck[3];
     struct rig_t rig;
     struct rt_t rt;
     struct interface_t iface;
     struct library_t library;
 
+#if defined WITH_OSS || WITH_ALSA
+    char *endptr;
+    int rate;
+#endif
+
+#ifdef WITH_OSS
+    int oss_buffers, oss_fragment;
+#endif
+
+#ifdef WITH_ALSA
+    int alsa_buffer;
+#endif
+
     fprintf(stderr, "%s\n\n" NOTICE "\n\n", banner);
 
     if (rig_init(&rig) == -1)
@@ -122,15 +131,24 @@ int main(int argc, char *argv[])
     library_init(&library);
 
     decks = 0;
-    oss_fragment = DEFAULT_OSS_FRAGMENT;
-    oss_buffers = DEFAULT_OSS_BUFFERS;
-    rate = DEFAULT_RATE;
-    alsa_buffer = DEFAULT_ALSA_BUFFER;
     importer = DEFAULT_IMPORTER;
     scanner = DEFAULT_SCANNER;
-    timecode = DEFAULT_TIMECODE;
+    timecode = NULL;
     speed = 1.0;
 
+#if defined WITH_OSS || WITH_ALSA
+    rate = DEFAULT_RATE;
+#endif
+
+#ifdef WITH_ALSA
+    alsa_buffer = DEFAULT_ALSA_BUFFER;
+#endif
+
+#ifdef WITH_OSS
+    oss_fragment = DEFAULT_OSS_FRAGMENT;
+    oss_buffers = DEFAULT_OSS_BUFFERS;
+#endif
+
     /* Skip over command name */
 
     argv++;
@@ -189,6 +207,7 @@ int main(int argc, char *argv[])
             argc -= 2;
 #endif
 
+#if defined WITH_OSS || WITH_ALSA
         } else if (!strcmp(argv[0], "-r")) {
 
             /* Set sample rate for subsequence devices */
@@ -206,6 +225,7 @@ int main(int argc, char *argv[])
 
             argv += 2;
             argc -= 2;
+#endif
 
 #ifdef WITH_ALSA
         } else if (!strcmp(argv[0], "-m")) {
@@ -231,6 +251,10 @@ int main(int argc, char *argv[])
 		  !strcmp(argv[0], "-j"))
 	{
             unsigned int sample_rate;
+            struct deck_t *ld;
+            struct device *device;
+            struct track_t *track;
+            struct timecoder_t *timecoder;
 
             /* Create a deck */
 
@@ -240,14 +264,18 @@ int main(int argc, char *argv[])
                 return -1;
             }
 
-            if (decks == MAX_DECKS) {
-                fprintf(stderr, "Too many decks (maximum %d); aborting.\n",
-                        MAX_DECKS);
+            if (decks == sizeof deck) {
+                fprintf(stderr, "Too many decks; aborting.\n");
                 return -1;
             }
 
             fprintf(stderr, "Initialising deck %d (%s)...\n", decks, argv[1]);
 
+            ld = &deck[decks];
+            device = &ld->device;
+            timecoder = &ld->timecoder;
+            track = &ld->track;
+
             /* Work out which device type we are using, and initialise
              * an appropriate device. */
 
@@ -255,18 +283,17 @@ int main(int argc, char *argv[])
 
 #ifdef WITH_OSS
             case 'd':
-                r = oss_init(&device[decks], argv[1], rate,
-                             oss_buffers, oss_fragment);
+                r = oss_init(device, argv[1], rate, oss_buffers, oss_fragment);
                 break;
 #endif
 #ifdef WITH_ALSA
             case 'a':
-                r = alsa_init(&device[decks], argv[1], rate, alsa_buffer);
+                r = alsa_init(device, argv[1], rate, alsa_buffer);
                 break;
 #endif
 #ifdef WITH_JACK
             case 'j':
-                r = jack_init(&device[decks], argv[1]);
+                r = jack_init(device, argv[1]);
                 break;
 #endif
             default:
@@ -280,27 +307,22 @@ int main(int argc, char *argv[])
 
 	    sample_rate = device_sample_rate(device);
 
-            if (timecoder_init(&timecoder[decks], timecode,
-                               speed, sample_rate) == -1)
-            {
-                return -1;
+            /* Default timecode decoder where none is specified */
+
+            if (timecode == NULL) {
+                timecode = timecoder_find_definition(DEFAULT_TIMECODE);
+                assert(timecode != NULL);
             }
 
-            if (rt_add_device(&rt, &device[decks]) == -1)
+            timecoder_init(timecoder, timecode, speed, sample_rate);
+            track_init(track, importer);
+
+            /* Connect up the elements to make an operational deck */
+
+            r = deck_init(ld, &rt, &rig);
+            if (r == -1)
                 return -1;
 
-            track_init(&track[decks], importer);
-            rig_add_track(&rig, &track[decks]);
-
-            player_init(&player[decks], &track[decks]);
-            player_connect_timecoder(&player[decks], &timecoder[decks]);
-
-            /* The timecoder and player are driven by requests from
-             * the audio device */
-
-            device_connect_timecoder(&device[decks], &timecoder[decks]);
-            device_connect_player(&device[decks], &player[decks]);
-
             decks++;
 
             argv += 2;
@@ -315,7 +337,11 @@ int main(int argc, char *argv[])
                 return -1;
             }
 
-            timecode = argv[1];
+            timecode = timecoder_find_definition(argv[1]);
+            if (timecode == NULL) {
+                fprintf(stderr, "Timecode '%s' is not known.\n", argv[1]);
+                return -1;
+            }
 
             argv += 2;
             argc -= 2;
@@ -394,15 +420,19 @@ int main(int argc, char *argv[])
         }
     }
 
+#ifdef WITH_ALSA
+    alsa_clear_config_cache();
+#endif
+
     if (decks == 0) {
         fprintf(stderr, "You need to give at least one audio device to use "
                 "as a deck; try -h.\n");
         return -1;
     }
 
-    if (rt_start(&rt) == -1)
+    if (interface_start(&iface, deck, decks, &library, &rig) == -1)
         return -1;
-    if (interface_start(&iface, decks, player, timecoder, &library, &rig) == -1)
+    if (rt_start(&rt) == -1)
         return -1;
 
     if (rig_main(&rig) == -1)
@@ -410,15 +440,11 @@ int main(int argc, char *argv[])
 
     fprintf(stderr, "Exiting cleanly...\n");
 
-    interface_stop(&iface);
     rt_stop(&rt);
+    interface_stop(&iface);
 
-    for (n = 0; n < decks; n++) {
-        track_clear(&track[n]);
-        timecoder_clear(&timecoder[n]);
-        player_clear(&player[n]);
-        device_clear(&device[n]);
-    }
+    for (n = 0; n < decks; n++)
+        deck_clear(&deck[n]);
 
     timecoder_free_lookup();
     library_clear(&library);
